 { -------------------------------------------------------------------------------------}
 { A "Registry" component for CodeGear Delphi.                                          }
 { Copyright 1998-2007, Jean-Fabien Connault.  All Rights Reserved.                     }
 { This component can be freely used and distributed in commercial and private          }
 { environments, provided this notice is not modified in any way.                       }
 { -------------------------------------------------------------------------------------}
 { Feel free to contact me if you have any questions, comments or suggestions at:       }
 {   cycocrew@wanadoo.fr (Jean-Fabien Connault)                                         }
 { You can always find the latest version of this component at:                         }
 {   http://perso.wanadoo.fr/cycocrew/delphi/components.html                            }
 { -------------------------------------------------------------------------------------}
 { Date last modified:  2007.05.20                                                      }
 { -------------------------------------------------------------------------------------}

 { -------------------------------------------------------------------------------------}
 { TRegistryEnhanced v1.45                                                              }
 { -------------------------------------------------------------------------------------}
 { Description:                                                                         }
 {   A component that allows you manipulate entries in the Registry                     }
 { Properties:                                                                          }
 {   property Access: TAccess;               "Registry access rights"                   }
 {   property BinaryType: TBinaryType;       "BinaryType to set/get binary type values" }
 {   property EntriesFound: TStringList;     "StringList to store the search result"    }
 {   property Key: String;                   "Key in Registry"                          }
 {   property LangType: TLangType;           "LangType"                                 }
 {   property RaiseExceptions: Boolean;      "To raise exceptions or not"               }
 {   property RootKey: TRootKey;             "RootKey"                                  }
 { Procedures and Functions:                                                            }
 {   function CopyKey(OldKey,NewKey:string): boolean;                                   }
 {   function CopyValue(OldValue,NewValue:string): boolean;                             }
 {   function CountSubKeys(ASubKey:string): integer;                                    }
 {   function CountValueNames(ASubKey:string): integer;                                 }
 {   function DeleteKey(AKey:string): boolean;                                          }
 {   function DeleteValue(AValue:string): boolean;                                      }
 {   procedure ExportKeyToRegFile(ASubKey:string;AFileName:string);                     }
 {   function GetCheckListBox(ASubKey:string;...): TCheckListBox                        }
 {   function GetComboBox(ASubKey:string;AComboBox:TComboBox;...): TComboBox;           }
 {   function GetFont(AValue:string): TFont;                                            }
 {   procedure GetFormInfo(AValue:string;AForm:TForm;CenterDefault:boolean);            }
 {   function GetMenu(ASubKey:string;AMenu:TMenu;AMenuClick:TNotifyEvent): TMenu;       }
 {   procedure GetSubKeys(ASubKey:string;AList:TStrings);                               }
 {   function GetValue(AValue:string): variant;                                         }
 {   function GetTypedValue(AValue:string;ABinaryType:TBinaryType): variant;            }
 {   function GetTypedValueDefault(AValue:string;...): variant                          }
 {   function GetValueDefault(AValue:string;AValueDataDefault:variant): variant;        }
 {   procedure GetValueNames(ASubKey:string;AList:TStrings);                            }
 {   procedure ImportFormInfo(ASubKey:string;AValue:string;DeleteSubKey:boolean);       }
 {   function KeyExists(ASubKey:string): boolean;                                       }
 {   procedure LoadKeyContentFromFile(ASubKey,AFilename,ABreakString: string);          }
 {   function RenameKey(OldKey,NewKey:string): boolean;                                 }
 {   function RenameValue(OldValue,NewValue:string): boolean;                           }
 {   procedure SaveKeyContentToFile(ASubKey,AFilename,ABreakString:string);             }
 {   procedure Search(ASubKey,AString:string;ASearchOption:TSearchOption;...);          }
 {   procedure SetCheckListBox(ASubKey:string;ACheckListBox:TCheckListBox);             }
 {   procedure SetComboBox(ASubKey:string;AComboBox:TComboBox;ShowText:boolean);        }
 {   procedure SetFont(AValue:string;AFont:TFont);                                      }
 {   procedure SetFormInfo(AValue:string;AForm:TForm);                                  }
 {   procedure SetMenu(ASubKey:string; AMenu:TMenu);                                    }
 {   procedure SetValue(AValue:string;AValueData:variant);                              }
 {   procedure SetTypedValue(AValue:string;AValueData:variant;...);                     }
 {   function ValueExists(AValue:string): boolean;                                      }
 { Needs:                                                                               }
 {   TBrkApart component from Patrick Brisacier and Jean-Fabien Connault                }
 { -------------------------------------------------------------------------------------}
 { See example contained in example.zip file for more details.                          }
 { -------------------------------------------------------------------------------------}
 { Revision History:                                                                    }
 { 1.00:  + Initial release                                                             }
 { 1.01:  + Fixed bug                                                                   }
 { 1.02:  + Added Registry search capabilities                                          }
 { 1.03:  + Fixed bug in GetValueDefault function                                       }
 { 1.04:  + Allowed an empty string as Key when using GetValueNames and GetSubKeys      }
 {          functions                                                                   }
 { 1.05:  + Read and Write now float values                                             }
 {        + Read now binary values                                                      }
 { 1.06:  + Added Access property to manage Registry access rights                      }
 {          Thanks to John of Fine Point Technologies (info@finepoint.com) for the add  }
 { 1.07:  + Added SetBinaryValue procedure to store binary values in the registry       }
 { 1.08:  + Added support of rdExpandString in GetValue and GetValueDefault functions   }
 { 1.09:  + Modified to work with Delphi 3                                              }
 { 1.10:  + Fixed bug in GetValue function related to binary data                       }
 {          Thanks to Arek (Arek2@aol.com) for the fix                                  }
 {        + Added KeyExists function                                                    }
 {          Thanks to Arek (Arek2@aol.com) for the add                                  }
 { 1.11:  + Fixed a bug in SetBinaryValue procedure                                     }
 {          Thanks to Arek (Arek2@aol.com) for the fix                                  }
 {        + Added GetBinaryValue function                                               }
 {          Thanks to Arek (Arek2@aol.com) for the add                                  }
 {        + Added SetBinValue procedure                                                 }
 {          Thanks to Arek (Arek2@aol.com) for the add                                  }
 { 1.12:  + Added languages (French and English) support                                }
 {        + Added SaveKeyContentToFile and LoadKeyContentFromFile procedures            }
 {          Note that there's no recursivity and it only applies to strings             }
 { 1.13:  + Fixed bug in SaveKeyContentToFile procedure                                 }
 { 1.14:  + Fixed bug in RenameKey procedure                                            }
 { 1.15:  + Language property changed to LangType property                              }
 {        + DeleteKey and DeleteValue procedures changed to functions                   }
 { 1.16:  + Fixed bug at initialization when LangType was equal to ltFrench             }
 { 1.17:  + Fixed bug in Search procedure                                               }
 { 1.18:  + Added SetFont procedure and GetFont function                                }
 { 1.19:  + Added ValueExists function                                                  }
 {        + Changed KeyExists function parameters                                       }
 {        + RenameKey and RenameValue procedures changed to functions                   }
 { 1.20:  + Modified to work with Delphi 4                                              }
 { 1.21:  + Added ExportKeyToRegFile procedure                                          }
 {          Thanks to Arthur Hoornweg (arthur.hoornweg@email.de) for the base code      }
 {        + Added support of HKEY_PERFORMANCE_DATA                                      }
 { 1.22:  + Keys are now systematically closed after being opened                       }
 { 1.23:  + Added CopyKey function                                                      }
 {        + Fixed return value in RenameKey and RenameValue functions                   }
 { 1.24:  + Changed ValueExists function parameters                                     }
 { 1.25:  + Modified to work with Delphi 5                                              }
 { 1.26:  + Minor fixes related to empty keys                                           }
 {          Thanks to Jim Davis (jimdavis2@primary.net) for the feedback                }
 { 1.27:  + Minor enhancements                                                          }
 { 1.30:  + Added exceptions handling                                                   }
 {        + Added BinaryType and RaiseExceptions properties                             }
 {        + Added CopyValue function                                                    }
 {        + Enhanced SetValue procedure                                                 }
 {        + Enhanced GetValue function                                                  }
 {        + Enhanced RenameValue function                                               }
 {        + Removed GetBinaryValue function                                             }
 {        + Removed SetBinValue and SetBinaryValue procedures                           }
 {        + Fixed Search procedure                                                      }
 { 1.31:  + Added strings encryption support                                            }
 {        + Added GetTypedValue and GetTypedValueDefault functions                      }
 {        + Added SetTypedValue and SetEncryptionParameters procedures                  }
 { 1.32:  + Added SetCheckListBox procedure and GetCheckListBox function                }
 {        + Enhanced SetFont procedure and GetFont function                             }
 { 1.33:  + Modified SetFont procedure and GetFont function for single string storage   }
 { 1.34:  + Changed default LangType value to ltEnglish (2001.02.09)                    }
 { 1.35:  + Added SetMenu procedure and GetMenu function (2001.05.06)                   }
 { 1.36:  + Fixed a compilation issue under Delphi 3 (2001.06.16)                       }
 {        + Modified to work with Delphi 6 (2001.06.16)                                 }
 {        + Changed Register page to "System" (2001.06.16)                              }
 { 1.40:  + Added CountSubKeys and CountValueNames functions (2001.08.03)               }
 {        + Added SetFormInfo, GetFormInfo and ImportFormInfo procedures (2001.08.03)   }
 {        + Added SetComboBox procedure and GetComboBox function (2001.08.03)           }
 { 1.41:  + Enhanced GetCheckListBox function (2001.08.19)                              }
 {        + Enhanced SetComboBox procedure and GetComboBox function (2001.08.19)        }
 {        + Enhanced GetFormInfo procedure (2001.08.19)                                 }
 { 1.42:  + Enhanced CloseKey procedure call (2001.11.21)                               }
 {        + Enhanced SetValue procedure (2001.11.21)                                    }
 {        + Fixed and optimized code for key management (2001.11.21)                    }
 { 1.43:  + Cosmetic changes (2002.06.30)                                               }
 { 1.44:  + Enhanced Search procedure (2004.07.13)                                      }
 { 1.45:  + Added support for CodeGear Delphi 2007 (2007.05.20)                         }
 {        + Removed support for Borland Delphi 2/3/4 (2007.05.20)                       }
 { -------------------------------------------------------------------------------------}

unit RegistryEnhanced;

interface

uses
  SysUtils, Windows, Classes, StdCtrls, Forms, Graphics, Variants,
  Registry, Dialogs, BrkApart, CheckLst, Menus, Controls, ComCtrls;

{$I-}
{$LONGSTRINGS ON}

type
  TRootKey    = (hkClassesRoot, hkCurrentConfig, hkCurrentUser, hkDynData,
    hkLocalMachine, hkPerformanceData, hkUsers);
  TSearchOption = (opValue, opData, opValueData, opKey);
  TAccess     = (kAllAccess, kExecute, kRead, kWrite);
  TLangType   = (ltFrench, ltEnglish);
  TBinaryType = (btNone, btEncryptString, btExpandString, btBinary32BitMax,
    btBinary32BitMin);
  ERegistryEnhanced = class(Exception);

  TRegistryEnhanced = class(TComponent)
  private
    FAccess:   TAccess;
    FBaseKey:  string;
    FBinaryType: TBinaryType;
    FDecryptKey: integer;
    FKey:      string;
    FLangType: TLangType;
    FRootKey:  TRootKey;
    FBrkApart: TBrkApart;
    FEntriesFound: TStringList;
    FRaiseExceptions: boolean;
    FBuffer:   array[1..2048] of byte;
    function CheckKey(AKey: string): string;
    function ExtractKeyPath(APath: string): string;
    function ExtractKeyValue(APath: string): string;
    function Encryption(Src: string; Encrypt: boolean): string;
    procedure SetLangType(ALangType: TLangType);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property EntriesFound: TStringList Read FEntriesFound Write FEntriesFound;
    procedure Search(ASubKey, AString: string; ASearchOption: TSearchOption;
      Recursive: boolean);
    procedure SetValue(AValue: string; AValueData: variant);
    procedure SetTypedValue(AValue: string; AValueData: variant;
      ABinaryType: TBinaryType);
    function GetValue(AValue: string): variant;
    function GetTypedValue(AValue: string; ABinaryType: TBinaryType): variant;
    function GetValueDefault(AValue: string; AValueDataDefault: variant): variant;
    function GetTypedValueDefault(AValue: string; AValueDataDefault: variant;
      ABinaryType: TBinaryType): variant;
    procedure GetValueNames(ASubKey: string; AList: TStrings);
    function CountValueNames(ASubKey: string): integer;
    procedure GetSubKeys(ASubKey: string; AList: TStrings);
    function CountSubKeys(ASubKey: string): integer;
    function DeleteKey(AKey: string): boolean;
    function DeleteValue(AValue: string): boolean;
    function CopyKey(OldKey, NewKey: string): boolean;
    function CopyValue(OldValue, NewValue: string): boolean;
    function RenameKey(OldKey, NewKey: string): boolean;
    function RenameValue(OldValue, NewValue: string): boolean;
    function KeyExists(ASubKey: string): boolean;
    function ValueExists(AValue: string): boolean;
    procedure SaveKeyContentToFile(ASubKey, AFilename, ABreakString: string);
    procedure LoadKeyContentFromFile(ASubKey, AFilename, ABreakString: string);
    procedure SetFont(AValue: string; AFont: TFont);
    function GetFont(AValue: string; AFont: TFont): TFont;
    procedure SetListViewInfo(AValue: string; AListView: TListView);
    procedure GetListViewInfo(AValue: string; AListView: TListView);
    procedure ImportListViewInfo(ASubKey: string; AValue: string; DeleteSubKey: boolean);
    procedure SetFormInfo(AValue: string; AForm: TForm);
    procedure GetFormInfo(AValue: string; AForm: TForm; CenterDefault: boolean = True);
    procedure ImportFormInfo(ASubKey: string; AValue: string; DeleteSubKey: boolean);
    procedure SetCheckListBox(ASubKey: string; ACheckListBox: TCheckListBox);
    function GetCheckListBox(ASubKey: string; ACheckListBox: TCheckListBox;
      CheckAllFirstTime: boolean = False): TCheckListBox;
    procedure SetMenu(ASubKey: string; AMenu: TMenu);
    function GetMenu(ASubKey: string; AMenu: TMenu; AMenuClick: TNotifyEvent): TMenu;
    procedure SetComboBox(ASubKey: string; AComboBox: TComboBox;
      ShowText: boolean = True);
    function GetComboBox(ASubKey: string; AComboBox: TComboBox;
      ShowText: boolean = True): TComboBox;
    procedure ExportKeyToRegFile(ASubKey: string; AFileName: string);
    procedure SetEncryptionParameters(ABaseKey: string; ADecryptKey: integer);
  published
    property Access: TAccess Read FAccess Write FAccess default kAllAccess;
    property BinaryType: TBinaryType Read FBinaryType Write FBinaryType default btNone;
    property Key: string Read FKey Write FKey;
    property LangType: TLangType Read FLangType Write SetLangType default ltEnglish;
    property RaiseExceptions: boolean Read FRaiseExceptions
      Write FRaiseExceptions default False;
    property RootKey: TRootKey Read FRootKey Write FRootKey default hkCurrentUser;
  end;

procedure Register;

const
  RootKeyValues: array[TRootKey] of DWORD =
    (HKEY_CLASSES_ROOT, HKEY_CURRENT_CONFIG, HKEY_CURRENT_USER, HKEY_DYN_DATA,
    HKEY_LOCAL_MACHINE, HKEY_PERFORMANCE_DATA, HKEY_USERS);
  AccessValues: array[TAccess] of integer =
    (KEY_ALL_ACCESS, KEY_EXECUTE, KEY_READ, KEY_WRITE);
  csfsBold      = '|Bold';
  csfsItalic    = '|Italic';
  csfsUnderline = '|Underline';
  csfsStrikeout = '|Strikeout';

var
  MSG_TOO_MANY_ENTRIES, MSG_UNKNOWN_DATA_TYPE, MSG_CANT_OPEN_FILE: string;

implementation

 {***************************************************************************}
 { CONSTRUCTOR Create                                                        }
 {***************************************************************************}

constructor TRegistryEnhanced.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAccess     := kAllAccess;
  FBaseKey    := '';
  FBinaryType := btNone;
  FDecryptKey := 0;
  FRootKey    := hkCurrentUser;
  FBrkApart   := TBrkApart.Create(Self);
  FEntriesFound := TStringList.Create;
  SetLangType(ltEnglish);
end;

 {***************************************************************************}
 { DESTRUCTOR Destroy                                                        }
 {***************************************************************************}

destructor TRegistryEnhanced.Destroy;
begin
  FBrkApart.Free;
  FEntriesFound.Free;
  inherited Destroy;
end;

 {***************************************************************************}
 { FUNCTION CheckKey                                                         }
 {***************************************************************************}

function TRegistryEnhanced.CheckKey(AKey: string): string;
begin
  if (length(Akey) > 0) and (AKey <> '\') then
  begin
    if (AKey[length(AKey)] = '\') then
      Result := copy(AKey, 1, length(AKey) - 1)
    else
      Result := AKey;
    if (AKey[1] <> '\') then
      Result := '\' + Result;
  end
  else
    Result := '';
end;

 {***************************************************************************}
 { FUNCTION ExtractKeyPath                                                   }
 {***************************************************************************}

function TRegistryEnhanced.ExtractKeyPath(APath: string): string;
begin
  FBrkApart.BaseString  := APath;
  FBrkApart.BreakString := '\';
  FBrkApart.AllowEmptyString := False;
  FBrkApart.BreakApart;
  if (FBrkApart.StringList.Count > 1) then
  begin
    FBrkApart.StringList.Delete(FBrkApart.StringList.Count - 1);
    FBrkApart.ReverseBreakApart;
    Result := '\' + FBrkApart.BaseString;
  end
  else
    Result := '';
end;

 {***************************************************************************}
 { FUNCTION ExtractKeyValue                                                  }
 {***************************************************************************}

function TRegistryEnhanced.ExtractKeyValue(APath: string): string;
begin
  FBrkApart.BaseString  := APath;
  FBrkApart.BreakString := '\';
  FBrkApart.AllowEmptyString := False;
  FBrkApart.BreakApart;
  if (FBrkApart.StringList.Count > 1) then
    Result := FBrkApart.StringList[FBrkApart.StringList.Count - 1]
  else
    Result := APath;
end;

 {*****************************************************************************}
 {* FUNCTION Encryption                                                       *}
 {*****************************************************************************}

function TRegistryEnhanced.Encryption(Src: string; Encrypt: boolean): string;
var
  KeyLen:    integer;
  KeyPos:    integer;
  Offset:    integer;
  Dest:      string;
  SrcPos:    integer;
  SrcAsc:    integer;
  TmpSrcAsc: integer;
begin
  KeyLen := Length(FBaseKey);
  KeyPos := 0;
  if (Encrypt = True) then
  begin
    // Encrypt
    Offset := FDecryptKey;
    Dest   := format('%1.2x', [Offset]);
    for SrcPos := 1 to length(Src) do
    begin
      SrcAsc := (Ord(Src[SrcPos]) + Offset) mod 255;
      if (KeyPos < KeyLen) then
        KeyPos := KeyPos + 1
      else
        KeyPos := 1;
      SrcAsc := SrcAsc xor Ord(FBaseKey[KeyPos]);
      Dest   := Dest + format('%1.2x', [SrcAsc]);
      Offset := SrcAsc;
    end;
  end
  else
  begin
    // Decrypt
    Offset := StrToInt('$' + copy(Src, 1, 2));
    SrcPos := 3;
    repeat
      SrcAsc := StrToInt('$' + copy(Src, SrcPos, 2));
      if (KeyPos < KeyLen) then
        KeyPos := KeyPos + 1
      else
        KeyPos := 1;
      TmpSrcAsc := SrcAsc xor Ord(FBaseKey[KeyPos]);
      if (TmpSrcAsc <= offset) then
        TmpSrcAsc := 255 + TmpSrcAsc - Offset
      else
        TmpSrcAsc := TmpSrcAsc - Offset;
      Dest := Dest + chr(TmpSrcAsc);
      Offset := SrcAsc;
      SrcPos := SrcPos + 2;
    until
      SrcPos >= Length(Src);
  end;
  Result := Dest;
end;

 {***************************************************************************}
 { PROCEDURE SetLangType                                                     }
 {***************************************************************************}

procedure TRegistryEnhanced.SetLangType(ALangType: TLangType);
begin
  FLangType := ALangType;

  if (ALangType = ltFrench) then
  begin
    MSG_TOO_MANY_ENTRIES  := 'Trop d''entres.';
    MSG_UNKNOWN_DATA_TYPE := 'Type de donne inconnu.';
    MSG_CANT_OPEN_FILE    := 'Impossible d''ouvrir le fichier.';
  end;

  if (ALangType = ltEnglish) then
  begin
    MSG_TOO_MANY_ENTRIES  := 'Too many entries.';
    MSG_UNKNOWN_DATA_TYPE := 'Unknown data type.';
    MSG_CANT_OPEN_FILE    := 'Can''t open file.';
  end;
end;

 {***************************************************************************}
 { FUNCTION KeyExists                                                        }
 {***************************************************************************}

function TRegistryEnhanced.KeyExists(ASubKey: string): boolean;
var
  Registry: TRegistry;
  AKey:     string;
begin
  Registry := TRegistry.Create;
  try
    Registry.Access := AccessValues[FAccess];
    Registry.RootKey := RootKeyValues[FRootKey];
    AKey   := CheckKey(FKey) + CheckKey(ASubKey);
    Result := Registry.KeyExists(AKey);
  finally
    Registry.Free;
  end;
end;

 {***************************************************************************}
 { FUNCTION ValueExists                                                      }
 {***************************************************************************}

function TRegistryEnhanced.ValueExists(AValue: string): boolean;
var
  Registry: TRegistry;
  AKey:     string;
begin
  Result   := False;
  Registry := TRegistry.Create;
  try
    Registry.Access := AccessValues[FAccess];
    Registry.RootKey := RootKeyValues[FRootKey];
    AKey := CheckKey(FKey) + ExtractKeyPath(AValue);
    if (Registry.OpenKey(AKey, False)) then
    begin
      Result := Registry.ValueExists(ExtractKeyValue(AValue));
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
end;

 {***************************************************************************}
 { PROCEDURE SetValue                                                        }
 {***************************************************************************}

procedure TRegistryEnhanced.SetValue(AValue: string; AValueData: variant);
var
  Registry: TRegistry;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), True)) then
      begin
        case VarType(AValueData) of
          varString:
            if (FBinaryType = btEncryptString) then
              Registry.WriteString(ExtractKeyValue(AValue), Encryption(AValueData, True))
            else
              Registry.WriteString(ExtractKeyValue(AValue), AValueData);
          varInteger: Registry.WriteInteger(ExtractKeyValue(AValue), AValueData);
          varBoolean: Registry.WriteBool(ExtractKeyValue(AValue), AValueData);
          varDate: Registry.WriteDateTime(ExtractKeyValue(AValue), AValueData);
          varCurrency: Registry.WriteCurrency(ExtractKeyValue(AValue), AValueData);
          varDouble: Registry.WriteFloat(ExtractKeyValue(AValue), AValueData);
          varByte: Registry.WriteInteger(ExtractKeyValue(AValue), AValueData);
          varSmallInt: Registry.WriteInteger(ExtractKeyValue(AValue), AValueData);
          varShortInt: Registry.WriteInteger(ExtractKeyValue(AValue), AValueData);
          varWord: Registry.WriteInteger(ExtractKeyValue(AValue), AValueData);
          varLongWord: Registry.WriteInteger(ExtractKeyValue(AValue), AValueData);
          else
            case FBinaryType of
              btNone:
                if (FRaiseExceptions = True) then
                  raise ERegistryEnhanced.Create(MSG_UNKNOWN_DATA_TYPE);
              btBinary32BitMax: Registry.WriteBinaryData(ExtractKeyValue(AValue),
                  AValueData, sizeof(integer)); // AValueData: integer
              btBinary32BitMin: Registry.WriteBinaryData(ExtractKeyValue(AValue),
                  AValueData, sizeof(AValueData)); // AValueData: array of byte
              btExpandString: Registry.WriteExpandString(ExtractKeyValue(AValue),
                  AValueData);
            end;
        end;
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE SetTypedValue                                                   }
 {***************************************************************************}

procedure TRegistryEnhanced.SetTypedValue(AValue: string; AValueData: variant;
  ABinaryType: TBinaryType);
begin
  FBinaryType := ABinaryType;
  SetValue(AValue, AValueData);
  FBinaryType := btNone;
end;

 {***************************************************************************}
 { FUNCTION GetValue                                                         }
 {***************************************************************************}

function TRegistryEnhanced.GetValue(AValue: string): variant;
var
  Registry:    TRegistry;
  Var1, Var2:  integer;
  AValueData:  variant;
  KeyDataInfo: TRegDataInfo;

  function IntExp(Var1: integer; var2: byte): integer;
  var
    X: byte;
    Y: integer;
  begin
    Y := 1;
    for X := 1 to Var2 do
      Y := Y * Var1;
    Result := Y;
  end;

begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), False)) then
      begin
        Registry.GetDataInfo(ExtractKeyValue(AValue), KeyDataInfo);
        case KeyDataInfo.RegData of
          rdString:
            if (FBinaryType = btEncryptString) then
              AValueData := Encryption(Registry.ReadString(
                ExtractKeyValue(AValue)), False)
            else
              AValueData := Registry.ReadString(ExtractKeyValue(AValue));
          rdInteger: AValueData := Registry.ReadInteger(ExtractKeyValue(AValue));
          rdBinary:
          begin
            case FBinaryType of
              btBinary32BitMax:
              begin
                // AValueData: integer
                Var1 := Registry.ReadBinaryData(ExtractKeyValue(AValue),
                  FBuffer, KeyDataInfo.DataSize);
                if (Var1 < 4) then
                  Var1 := 4;
                if (Var1 > 0) then
                begin
                  AValueData := 0;
                  for Var2 := 1 to var1 do
                  begin
                    if (Var2 = 1) then
                      AValueData := FBuffer[1]
                    else
                      AValueData :=
                        AValueData + (FBuffer[Var2] * IntExp(256, Var2 - 1));
                  end;
                end;
              end;
              btBinary32BitMin:
              begin
                // AValueData: array of byte
                Registry.ReadBinaryData(ExtractKeyValue(AValue),
                  FBuffer, KeyDataInfo.DataSize);
                AValueData := FBuffer[1];
              end;
            end;
          end;
          rdExpandString: AValueData := Registry.ReadString(ExtractKeyValue(AValue));
          else
            if (FRaiseExceptions = True) and Registry.ValueExists(
              ExtractKeyValue(AValue)) then
              raise ERegistryEnhanced.Create(MSG_UNKNOWN_DATA_TYPE);
        end;
        Result := AValueData;
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION GetTypedValue                                                    }
 {***************************************************************************}

function TRegistryEnhanced.GetTypedValue(AValue: string;
  ABinaryType: TBinaryType): variant;
begin
  FBinaryType := ABinaryType;
  Result      := GetValue(AValue);
  FBinaryType := btNone;
end;

 {***************************************************************************}
 { FUNCTION GetValueDefault                                                  }
 {***************************************************************************}

function TRegistryEnhanced.GetValueDefault(AValue: string;
  AValueDataDefault: variant): variant;
var
  Registry:    TRegistry;
  AValueData:  variant;
  KeyDataInfo: TRegDataInfo;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), False)) then
      begin
        Registry.GetDataInfo(ExtractKeyValue(AValue), KeyDataInfo);
        case KeyDataInfo.RegData of
          rdString:
            try
              if (FBinaryType = btEncryptString) then
                AValueData :=
                  Encryption(Registry.ReadString(ExtractKeyValue(AValue)), False)
              else
                AValueData := Registry.ReadString(ExtractKeyValue(AValue));
            except
              AValueData := AValueDataDefault;
            end;
          rdInteger:
            try
              AValueData := Registry.ReadInteger(ExtractKeyValue(AValue));
            except
              AValueData := AValueDataDefault;
            end;
          rdBinary:
            try
              AValueData := Registry.ReadBinaryData(ExtractKeyValue(AValue),
                FBuffer, KeyDataInfo.DataSize);
            except
              AValueData := AValueDataDefault;
            end;
          rdExpandString:
            try
              AValueData := Registry.ReadString(ExtractKeyValue(AValue));
            except
              AValueData := AValueDataDefault;
            end;
          else
            if (FRaiseExceptions = True) and
              (Registry.ValueExists(ExtractKeyValue(AValue))) then
              raise ERegistryEnhanced.Create(MSG_UNKNOWN_DATA_TYPE);
            AValueData := AValueDataDefault;
        end;
        Result := AValueData;
        Registry.CloseKey;
      end
      else
        Result := AValueDataDefault;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION GetTypedValueDefault                                             }
 {***************************************************************************}

function TRegistryEnhanced.GetTypedValueDefault(AValue: string;
  AValueDataDefault: variant; ABinaryType: TBinaryType): variant;
begin
  FBinaryType := ABinaryType;
  Result      := GetValueDefault(AValue, AValueDataDefault);
  FBinaryType := btNone;
end;

 {***************************************************************************}
 { PROCEDURE GetValueNames                                                   }
 {***************************************************************************}

procedure TRegistryEnhanced.GetValueNames(ASubKey: string; AList: TStrings);
var
  Registry: TRegistry;
  AKey:     string;
begin
  Registry := TRegistry.Create;
  try
    Registry.Access := AccessValues[FAccess];
    Registry.RootKey := RootKeyValues[FRootKey];
    AKey := CheckKey(FKey) + CheckKey(ASubKey);
    if (Registry.OpenKey(AKey, False)) then
    begin
      Registry.GetValueNames(AList);
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
end;

 {***************************************************************************}
 { FUNCTION CountValueNames                                                  }
 {***************************************************************************}

function TRegistryEnhanced.CountValueNames(ASubKey: string): integer;
var
  AList: TStrings;
begin
  AList := TStringList.Create;
  GetValueNames(ASubKey, AList);
  Result := AList.Count;
  AList.Free;
end;

 {***************************************************************************}
 { PROCEDURE GetSubKeys                                                      }
 {***************************************************************************}

procedure TRegistryEnhanced.GetSubKeys(ASubKey: string; AList: TStrings);
var
  Registry: TRegistry;
  AKey:     string;
begin
  Registry := TRegistry.Create;
  try
    Registry.Access := AccessValues[FAccess];
    Registry.RootKey := RootKeyValues[FRootKey];
    AKey := CheckKey(FKey) + CheckKey(ASubKey);
    if (Registry.OpenKey(AKey, False)) then
    begin
      Registry.GetKeyNames(AList);
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
end;

 {***************************************************************************}
 { FUNCTION CountSubKeys                                                     }
 {***************************************************************************}

function TRegistryEnhanced.CountSubKeys(ASubKey: string): integer;
var
  AList: TStrings;
begin
  AList := TStringList.Create;
  GetSubKeys(ASubKey, AList);
  Result := AList.Count;
  AList.Free;
end;

 {***************************************************************************}
 { FUNCTION DeleteKey                                                        }
 {***************************************************************************}

function TRegistryEnhanced.DeleteKey(AKey: string): boolean;
var
  Registry: TRegistry;
begin
  Result := False;
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey), False)) then
      begin
        Result := Registry.DeleteKey(AKey);
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION DeleteValue                                                      }
 {***************************************************************************}

function TRegistryEnhanced.DeleteValue(AValue: string): boolean;
var
  Registry: TRegistry;
begin
  Result := False;
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), False)) then
      begin
        Result := Registry.DeleteValue(ExtractKeyValue(AValue));
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION CopyKey                                                          }
 {***************************************************************************}

function TRegistryEnhanced.CopyKey(OldKey, NewKey: string): boolean;
var
  Registry: TRegistry;
begin
  Result := False;
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if not (Registry.OpenKey(CheckKey(FKey) + CheckKey(NewKey), False)) then
      begin
        if (Registry.OpenKey(CheckKey(FKey), False)) then
        begin
          Registry.MoveKey(OldKey, NewKey, False);
          Registry.CloseKey;
        end;
        if (Registry.OpenKey(CheckKey(FKey) + CheckKey(NewKey), False)) then
        begin
          Result := True;
          Registry.CloseKey;
        end;
      end
      else
      begin
        Result := False;
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION CopyValue                                                        }
 {***************************************************************************}

function TRegistryEnhanced.CopyValue(OldValue, NewValue: string): boolean;
var
  Registry:    TRegistry;
  AValueData:  variant;
  KeyDataInfo: TRegDataInfo;
begin
  Result := False;
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(OldValue), False)) then
      begin
        Registry.GetDataInfo(ExtractKeyValue(OldValue), KeyDataInfo);
        case KeyDataInfo.RegData of
          rdString: AValueData  := Registry.ReadString(ExtractKeyValue(OldValue));
          rdInteger: AValueData := Registry.ReadInteger(ExtractKeyValue(OldValue));
          rdExpandString: AValueData := Registry.ReadString(ExtractKeyValue(OldValue));
          else
            if (FRaiseExceptions = True) then
              raise ERegistryEnhanced.Create(MSG_UNKNOWN_DATA_TYPE);
        end;
        if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(NewValue), True)) then
        begin
          case KeyDataInfo.RegData of
            rdString: Registry.WriteString(ExtractKeyValue(NewValue), AValueData);
            rdInteger: Registry.WriteInteger(ExtractKeyValue(NewValue), AValueData);
            rdExpandString: Registry.WriteExpandString(ExtractKeyValue(NewValue),
                AValueData);
          end;
          Result := Registry.ValueExists(ExtractKeyValue(NewValue));
          Registry.CloseKey;
        end;
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION RenameKey                                                        }
 {***************************************************************************}

function TRegistryEnhanced.RenameKey(OldKey, NewKey: string): boolean;
var
  Registry: TRegistry;
begin
  Result := False;
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if not (Registry.OpenKey(CheckKey(FKey) + CheckKey(NewKey), False)) then
      begin
        if (Registry.OpenKey(CheckKey(FKey), False)) then
        begin
          Registry.MoveKey(OldKey, NewKey, True);
          Registry.CloseKey;
        end;
        if (Registry.OpenKey(CheckKey(FKey) + CheckKey(NewKey), False)) then
        begin
          Result := True;
          Registry.CloseKey;
        end;
      end
      else
        Registry.CloseKey;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION RenameValue                                                      }
 {***************************************************************************}

function TRegistryEnhanced.RenameValue(OldValue, NewValue: string): boolean;
var
  Registry: TRegistry;
begin
  Result := False;
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (ExtractKeyPath(OldValue) = ExtractKeyPath(NewValue)) then
      begin
        if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(NewValue), False)) then
        begin
          if not (Registry.ValueExists(ExtractKeyValue(NewValue))) then
          begin
            if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(OldValue), False)) then
            begin
              Registry.RenameValue(extractKeyValue(OldValue), ExtractKeyValue(NewValue));
              if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(NewValue),
                False)) then
              begin
                Result := Registry.ValueExists(ExtractKeyValue(NewValue));
                Registry.CloseKey;
              end;
              Registry.CloseKey;
            end;
          end;
          Registry.CloseKey;
        end;
      end
      else
      begin
        if (CopyValue(OldValue, NewValue)) then
          Result := DeleteValue(OldValue);
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE Search                                                          }
 {***************************************************************************}
 { ASubKey: SubKey of search                                                 }
 { AString: String to search                                                 }
 { ASearchOption: Search Option                                              }
 {     + opValue:        Search in Value only                                }
 {     + opData:         Search in Data only                                 }
 {     + opValueData:    Search in Value and Data                            }
 {     + opKey:          Search in Key only                                  }
 {***************************************************************************}

procedure TRegistryEnhanced.Search(ASubKey, AString: string;
  ASearchOption: TSearchOption; Recursive: boolean);
var
  Registry: TRegistry;
  AKey:     string;

  procedure ProcessBranch(AKey: string);
  var
    tslValues, tslKeys: TStringList;
    DataString: string;
    i:    longint;
    flag: boolean;
  begin
    Registry.OpenKey(AKey, False);
    tslKeys := TStringList.Create;
    Registry.GetKeyNames(tslKeys);

    if (ASearchOption <> opKey) then
    begin
      tslValues := TStringList.Create;
      Registry.GetValueNames(tslValues);
      for i := 0 to tslValues.Count - 1 do
      begin
        if (AString = '') then
          flag := True
        else
        begin
          flag := False;
          case Registry.GetDataType(tslValues[i]) of
            rdString, rdExpandString:
              DataString := Registry.ReadString(tslValues[i]);
            rdInteger:
              DataString := IntToStr(Registry.ReadInteger(tslValues[i]));
          end;
          case ASearchOption of
            opValue:
              if (pos(lowercase(AString), lowerCase(tslValues[i])) > 0) then
                flag := True;
            opData:
              if (pos(lowercase(AString), lowercase(DataString)) > 0) then
                flag := True;
            opValueData:
              if (pos(lowercase(AString), lowerCase(tslValues[i])) > 0) or
                (pos(lowercase(AString), lowercase(DataString)) > 0) then
                flag := True;
          end;
        end;
        if (flag = True) then
        begin
          try
            FEntriesFound.add(AKey + '\' + tslValues[i]);
          except
            on EOutOfResources do
              Abort;
          end;
        end;
      end;
      tslValues.Free;
    end;

    Registry.CloseKey;

    for i := 0 to tslKeys.Count - 1 do
    begin
      if (ASearchOption = opKey) then
      begin
        if (pos(lowercase(AString), lowerCase(tslKeys[i])) > 0) or (AString = '') then
        begin
          try
            FEntriesFound.add(AKey + '\' + tslKeys[i]);
          except
            on EOutOfResources do
              Abort;
          end;
        end;
      end;
      if (Recursive = True) then
        ProcessBranch(AKey + '\' + tslKeys[i]);
    end;
    tslKeys.Free;
  end;

begin
  if (FKey <> '') then
  begin
    FEntriesFound.Clear;
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      ProcessBranch(AKey);
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE SaveKeyContentToFile (String and Integer Values Data only)      }
 {***************************************************************************}
 { ASubKey: SubKey content to be saved                                       }
 { AFilename: Filename where SubKey content will be saved to                 }
 { ABreakString: Break String (aka "Separator")                              }
 {***************************************************************************}

procedure TRegistryEnhanced.SaveKeyContentToFile(ASubKey, AFilename,
  ABreakString: string);
var
  Registry, RegistryValue: TRegistry;
  AKey, AValue, AValueData: string;
  tslValueNames, tslFileName: TStringList;
  KeyDataInfo: TRegDataInfo;
  i: integer;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      if (Registry.OpenKey(AKey, False)) then
      begin
        tslValueNames := TStringList.Create;
        Registry.GetValueNames(tslValueNames);
        tslFileName := TStringList.Create;
        for i := 0 to tslValueNames.Count - 1 do
        begin
          AValue := tslValueNames[i];
          RegistryValue := TRegistry.Create;
          try
            RegistryValue.Access  := AccessValues[FAccess];
            RegistryValue.RootKey := RootKeyValues[FRootKey];
            if (RegistryValue.OpenKey(AKey + ExtractKeyPath(AValue), False)) then
            begin
              RegistryValue.GetDataInfo(ExtractKeyValue(AValue), KeyDataInfo);
              case KeyDataInfo.RegData of
                rdString: AValueData  :=
                    RegistryValue.ReadString(ExtractKeyValue(AValue));
                rdInteger: AValueData :=
                    IntToStr(RegistryValue.ReadInteger(ExtractKeyValue(AValue)));
                rdExpandString: AValueData :=
                    RegistryValue.ReadString(ExtractKeyValue(AValue));
              end;
              RegistryValue.CloseKey;
            end;
          finally
            RegistryValue.Free;
          end;
          tslFileName.add(AValue + ABreakString + AValueData);
        end;
        try
          tslFileName.SaveToFile(AFileName);
        except
          if (FRaiseExceptions = True) then
            raise ERegistryEnhanced.Create(MSG_CANT_OPEN_FILE);
        end;
        tslFileName.Free;
        tslValueNames.Free;
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE LoadKeyContentFromFile (String Values Data only)                }
 {***************************************************************************}
 { ASubKey: SubKey content to be loaded                                      }
 { AFilename: Filename where SubKey content will be loaded from              }
 { ABreakString: Break String (aka "Separator")                              }
 {***************************************************************************}

procedure TRegistryEnhanced.LoadKeyContentFromFile(ASubKey, AFilename,
  ABreakString: string);
var
  Registry: TRegistry;
  AKey, AValue, AValueData: string;
  tslFileName: TStringList;
  i: integer;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      tslFileName := TStringList.Create;
      try
        tslFileName.LoadFromFile(AFileName);
      except
        if (FRaiseExceptions = True) then
          raise ERegistryEnhanced.Create(MSG_CANT_OPEN_FILE);
      end;
      for i := 0 to tslFileName.Count - 1 do
      begin
        FBrkApart.BaseString  := tslFileName[i];
        FBrkApart.BreakString := ABreakString;
        FBrkApart.AllowEmptyString := True;
        FBrkApart.BreakApart;
        if (FBrkApart.StringList.Count = 2) then
        begin
          AValue     := FBrkApart.StringList[0];
          AValueData := FBrkApart.StringList[1];
          if (Registry.OpenKey(AKey, True)) then
          begin
            Registry.WriteString(ExtractKeyValue(AValue), AValueData);
            Registry.CloseKey;
          end;
        end;
      end;
      tslFileName.Free;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE SetFont                                                         }
 {***************************************************************************}

procedure TRegistryEnhanced.SetFont(AValue: string; AFont: TFont);
var
  Registry: TRegistry;

  function FontToString(AFont: TFont): string;
  var
    sStyle: string;
  begin
    with AFont do
    begin
      sStyle := '';
      if (fsBold in Style) then
        sStyle := sStyle + csfsBold;
      if (fsItalic in Style) then
        sStyle := sStyle + csfsItalic;
      if (fsUnderline in Style) then
        sStyle := sStyle + csfsUnderline;
      if (fsStrikeout in Style) then
        sStyle := sStyle + csfsStrikeout;
      if ((Length(sStyle) > 0) and ('|' = sStyle[1])) then
        sStyle := Copy(sStyle, 2, Length(sStyle) - 1);
      Result := Format('"%s", %d, [%s], [%s]', [Name, Size, sStyle,
        ColorToString(Color)]);
    end;
  end;

begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), True)) then
      begin
        Registry.WriteString(ExtractKeyValue(AValue), FontToString(AFont));
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION GetFont                                                          }
 {***************************************************************************}

function TRegistryEnhanced.GetFont(AValue: string; AFont: TFont): TFont;
var
  Registry: TRegistry;

  function StringToFont(sFont: string; AFont: TFont): TFont;
  var
    p:      integer;
    sStyle: string;
  begin
    with AFont do
    begin
      // Get font name
      p    := Pos(',', sFont);
      Name := Copy(sFont, 2, p - 3);
      Delete(sFont, 1, p);
      // Get font size
      p    := Pos(',', sFont);
      Size := StrToInt(Copy(sFont, 2, p - 2));
      Delete(sFont, 1, p);
      // Get font style
      p      := Pos(',', sFont);
      sStyle := '|' + Copy(sFont, 3, p - 4);
      Delete(sFont, 1, p);
      // Get font color
      Color := StringToColor(Copy(sFont, 3, Length(sFont) - 3));
      // Convert str font style to font style
      Style := [];
      if (Pos(csfsBold, sStyle) > 0) then
        Style := Style + [fsBold];
      if (Pos(csfsItalic, sStyle) > 0) then
        Style := Style + [fsItalic];
      if (Pos(csfsUnderline, sStyle) > 0) then
        Style := Style + [fsUnderline];
      if (Pos(csfsStrikeout, sStyle) > 0) then
        Style := Style + [fsStrikeout];
    end;
    Result := AFont;
  end;

begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), False)) then
      begin
        if (Registry.ReadString(ExtractKeyValue(AValue)) <> '') then
          AFont := StringToFont(Registry.ReadString(ExtractKeyValue(AValue)), AFont);
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
  Result := AFont;
end;

 {***************************************************************************}
 { PROCEDURE SetListViewInfo                                                 }
 {***************************************************************************}

procedure TRegistryEnhanced.SetListViewInfo(AValue: string; AListView: TListView);
var
  Registry: TRegistry;

  function ListViewInfoToString(AListView: TListView): string;
  var
    i: integer;
  begin
    Result := '';
    for i := 0 to AListView.Columns.Count - 1 do
    begin
      Result := Result + IntToStr(AListView.Columns.Items[i].Width);
      if (i < AListView.Columns.Count - 1) then
        Result := Result + ', ';
    end;
  end;

begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), True)) then
      begin
        Registry.WriteString(ExtractKeyValue(AValue), ListViewInfoToString(AListView));
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE GetListViewInfo                                                 }
 {***************************************************************************}

procedure TRegistryEnhanced.GetListViewInfo(AValue: string; AListView: TListView);
var
  Registry: TRegistry;

  procedure StringToListViewInfo(sListViewInfo: string; AListView: TListView);
  var
    i: integer;
  begin
    FBrkApart.BaseString  := sListViewInfo;
    FBrkApart.BreakString := ', ';
    FBrkApart.AllowEmptyString := True;
    FBrkApart.BreakApart;
    for i := 0 to FBrkApart.StringList.Count - 1 do
      AListView.Columns.Items[i].Width := StrToInt(FBrkApart.StringList[i]);
  end;

begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), False)) then
      begin
        if (Registry.ReadString(ExtractKeyValue(AValue)) <> '') then
          StringToListViewInfo(Registry.ReadString(ExtractKeyValue(AValue)), AListView);
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE ImportListViewInfo                                              }
 {***************************************************************************}

procedure TRegistryEnhanced.ImportListViewInfo(ASubKey: string;
  AValue: string; DeleteSubKey: boolean);
var
  sListViewInfo: string;

  function ReformatString(sListViewInfo: string): string;
  begin
    FBrkApart.BaseString  := sListViewInfo;
    FBrkApart.BreakString := ',';
    FBrkApart.AllowEmptyString := True;
    FBrkApart.BreakApart;
    FBrkApart.BreakString := ', ';
    FBrkApart.ReverseBreakApart;
    Result := FBrkApart.BaseString;
  end;

begin
  if (KeyExists(ASubKey) = True) then
  begin
    sListViewInfo := GetValueDefault(CheckKey(ASubKey) + '\Columns\Sizes', '');
    if (DeleteSubKey = True) then
      DeleteKey(ASubKey);
    SetValue(AValue, ReformatString(sListViewInfo));
  end;
end;

 {***************************************************************************}
 { PROCEDURE SetFormInfo                                                     }
 {***************************************************************************}

procedure TRegistryEnhanced.SetFormInfo(AValue: string; AForm: TForm);
var
  Registry: TRegistry;

  function FormInfoToString(AForm: TForm): string;
  begin
    Result := IntToStr(AForm.Height) + ', ' + IntToStr(AForm.Left) +
      ', ' + IntToStr(AForm.Top) + ', ' + IntToStr(AForm.Width) + ', ';
    if (AForm.WindowState = wsMaximized) then
      Result := Result + 'wsMaximized';
    if (AForm.WindowState = wsMinimized) then
      Result := Result + 'wsMinimized';
    if (AForm.WindowState = wsNormal) then
      Result := Result + 'wsNormal';
  end;

begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), True)) then
      begin
        Registry.WriteString(ExtractKeyValue(AValue), FormInfoToString(AForm));
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE GetFormInfo                                                     }
 {***************************************************************************}

procedure TRegistryEnhanced.GetFormInfo(AValue: string; AForm: TForm;
  CenterDefault: boolean);
var
  Registry: TRegistry;

  procedure StringToFormInfo(sFormInfo: string; AForm: TForm);
  begin
    FBrkApart.BaseString  := sFormInfo;
    FBrkApart.BreakString := ', ';
    FBrkApart.AllowEmptyString := True;
    FBrkApart.BreakApart;
    if (FBrkApart.StringList.Count = 5) then
    begin
      AForm.Height := StrToInt(FBrkApart.StringList[0]);
      AForm.Left   := StrToInt(FBrkApart.StringList[1]);
      AForm.Top    := StrToInt(FBrkApart.StringList[2]);
      AForm.Width  := StrToInt(FBrkApart.StringList[3]);
      if (FBrkApart.StringList[4] = 'wsMaximized') then
        AForm.WindowState := wsMaximized;
      if (FBrkApart.StringList[4] = 'wsMinimized') then
        AForm.WindowState := wsMinimized;
      if (FBrkApart.StringList[4] = 'wsNormal') then
        AForm.WindowState := wsNormal;
    end;
  end;

  procedure CenterForm(AForm: TForm; CenterDefault: boolean);
  begin
    if (CenterDefault = True) then
    begin
      AForm.Left := (Screen.Width - AForm.Width) div 2;
      AForm.Top  := (Screen.Height - AForm.Height) div 2;
    end;
  end;

begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access  := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      if (Registry.OpenKey(CheckKey(FKey) + ExtractKeyPath(AValue), False)) then
      begin
        if (AForm.Position <> poDesigned) then
          MessageDlg('This warning indicates that position of ' +
            AForm.ClassName + ' will not be restored as ' + AForm.ClassName +
            '.Position <> poDesigned.', mtWarning, [mbOK], 0);
        if (Registry.ReadString(ExtractKeyValue(AValue)) <> '') then
          StringToFormInfo(Registry.ReadString(ExtractKeyValue(AValue)), AForm)
        else
          CenterForm(AForm, CenterDefault);
        Registry.CloseKey;
      end
      else
        CenterForm(AForm, CenterDefault);
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE ImportFormInfo                                                  }
 {***************************************************************************}

procedure TRegistryEnhanced.ImportFormInfo(ASubKey: string; AValue: string;
  DeleteSubKey: boolean);
var
  sFormInfo: string;
begin
  if (KeyExists(ASubKey) = True) then
  begin
    sFormInfo := GetValueDefault(CheckKey(ASubKey) + '\Height', '0') +
      ', ' + GetValueDefault(CheckKey(ASubKey) + '\Left', '0') +
      ', ' + GetValueDefault(CheckKey(ASubKey) + '\Top', '0') + ', ' +
      GetValueDefault(CheckKey(ASubKey) + '\Width', '0') + ', ';
    if (GetValueDefault(CheckKey(ASubKey) + '\WindowState', '1') = '0') then
      sFormInfo := sFormInfo + 'wsMaximized'
    else
      sFormInfo := sFormInfo + 'wsNormal';
    if (DeleteSubKey = True) then
      DeleteKey(ASubKey);
    SetValue(AValue, sFormInfo);
  end;
end;

 {***************************************************************************}
 { PROCEDURE SetCheckListBox                                                 }
 {***************************************************************************}

procedure TRegistryEnhanced.SetCheckListBox(ASubKey: string;
  ACheckListBox: TCheckListBox);
var
  Registry: TRegistry;
  AKey: string;
  i: integer;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      Registry.DeleteKey(AKey);
      if (Registry.OpenKey(AKey, True)) then
      begin
        for i := 0 to ACheckListBox.items.Count - 1 do
          Registry.WriteBool(ACheckListBox.items[i], ACheckListBox.Checked[i]);
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION GetCheckListBox                                                  }
 {***************************************************************************}

function TRegistryEnhanced.GetCheckListBox(ASubKey: string;
  ACheckListBox: TCheckListBox; CheckAllFirstTime: boolean): TCheckListBox;
var
  Registry: TRegistry;
  AKey: string;
  i: integer;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      if (Registry.OpenKey(AKey, False)) then
      begin
        Registry.GetValueNames(ACheckListBox.items);
        for i := 0 to ACheckListBox.items.Count - 1 do
          ACheckListBox.Checked[i] := Registry.ReadBool(ACheckListBox.items[i]);
        Registry.CloseKey;
      end
      else
      begin
        if (CheckAllFirstTime = True) then
        begin
          for i := 0 to ACheckListBox.items.Count - 1 do
            ACheckListBox.Checked[i] := True;
        end;
      end;
    finally
      Registry.Free;
    end;
  end;
  Result := ACheckListBox;
end;

 {***************************************************************************}
 { PROCEDURE SetMenu                                                         }
 {***************************************************************************}

procedure TRegistryEnhanced.SetMenu(ASubKey: string; AMenu: TMenu);
var
  Registry: TRegistry;
  AKey: string;
  i: integer;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      Registry.DeleteKey(AKey);
      if (Registry.OpenKey(AKey, True)) then
      begin
        Registry.WriteInteger('Count', AMenu.items.Count);
        for i := 0 to AMenu.items.Count - 1 do
          Registry.WriteString('Item' + IntToStr(i), AMenu.items[i].Caption);
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION GetMenu                                                          }
 {***************************************************************************}

function TRegistryEnhanced.GetMenu(ASubKey: string; AMenu: TMenu;
  AMenuClick: TNotifyEvent): TMenu;
var
  Registry:  TRegistry;
  AKey:      string;
  i, Count:  integer;
  AMenuItem: TMenuItem;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      if (Registry.OpenKey(AKey, False)) then
      begin
        AMenu.items.Clear;
        Count := Registry.ReadInteger('Count');
        for i := 0 to Count - 1 do
        begin
          AMenuItem := TMenuItem.Create(nil);
          AMenuItem.Caption := Registry.ReadString('Item' + IntToStr(i));
          AMenuItem.OnClick := AMenuClick;
          AMenu.items.add(AMenuItem);
        end;
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
  Result := AMenu;
end;

 {***************************************************************************}
 { PROCEDURE SetComboBox                                                     }
 {***************************************************************************}

procedure TRegistryEnhanced.SetComboBox(ASubKey: string; AComboBox: TComboBox;
  ShowText: boolean);
var
  Registry: TRegistry;
  AKey: string;
  i: integer;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      Registry.DeleteKey(AKey);
      if (Registry.OpenKey(AKey, True)) then
      begin
        Registry.WriteInteger('Count', AComboBox.items.Count);
        if (ShowText = True) then
          Registry.WriteString('Text', AComboBox.Text)
        else
          Registry.WriteString('Text', '');
        for i := 0 to AComboBox.items.Count - 1 do
          Registry.WriteString('Item' + IntToStr(i), AComboBox.items[i]);
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
end;

 {***************************************************************************}
 { FUNCTION GetComboBox                                                      }
 {***************************************************************************}

function TRegistryEnhanced.GetComboBox(ASubKey: string; AComboBox: TComboBox;
  ShowText: boolean): TComboBox;
var
  Registry: TRegistry;
  AKey:     string;
  i, Count: integer;
begin
  if (FKey <> '') then
  begin
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      if (Registry.OpenKey(AKey, False)) then
      begin
        AComboBox.items.Clear;
        if (ShowText = True) then
          AComboBox.Text := Registry.ReadString('Text')
        else
          AComboBox.Text := '';
        Count := Registry.ReadInteger('Count');
        for i := 0 to Count - 1 do
          AComboBox.items.add(Registry.ReadString('Item' + IntToStr(i)));
        Registry.CloseKey;
      end;
    finally
      Registry.Free;
    end;
  end;
  Result := AComboBox;
end;

 {***************************************************************************}
 { PROCEDURE ExportKeyToRegFile                                              }
 {***************************************************************************}

procedure TRegistryEnhanced.ExportKeyToRegFile(ASubKey: string; AFileName: string);
var
  Registry: TRegistry;
  AKey: string;
  F: TextFile;
  p: PChar;

  procedure ProcessBranch(AKey: string);
  var
    tslValues, tslKeys: TStringList;
    i, j, k: longint;
    s, t:    string;

    function DblBackSlash(AString: string): string;
    var
      k: longint;
    begin
      Result := AString;
      for k := length(AString) downto 1 do
        if (Result[k] = '\') then
          insert('\', Result, k);
    end;

  begin
    Writeln(F);
    case RootKeyValues[FRootKey] of
      HKEY_CLASSES_ROOT: s := 'HKEY_CLASSES_ROOT';
      HKEY_CURRENT_USER: s := 'HKEY_CURRENT_USER';
      HKEY_LOCAL_MACHINE: s := 'HKEY_LOCAL_MACHINE';
      HKEY_USERS: s    := 'HKEY_USERS';
      HKEY_PERFORMANCE_DATA: s := 'HKEY_PERFORMANCE_DATA';
      HKEY_CURRENT_CONFIG: s := 'HKEY_CURRENT_CONFIG';
      HKEY_DYN_DATA: s := 'HKEY_DYN_DATA';
    end;
    Writeln(F, '[' + s + AKey + ']');
    Registry.OpenKey(AKey, False);
    tslValues := TStringList.Create;
    tslKeys   := TStringList.Create;
    Registry.GetValueNames(tslValues);
    Registry.GetKeyNames(tslKeys);
    for i := 0 to tslValues.Count - 1 do
    begin
      s := tslValues[i];
      t := s;
      if (s = '') then
        s := '@'
      else
        s := '"' + s + '"';
      Write(F, DblBackSlash(s) + '=');
      case Registry.GetDataType(t) of
        rdString, rdExpandString:
          Writeln(F, '"' + DblBackSlash(Registry.ReadString(t) + '"'));
        rdInteger:
          Writeln(F, 'dword:' + IntToHex(Registry.ReadInteger(t), 8));
        rdBinary:
        begin
          Write(F, 'hex:');
          j := Registry.GetDataSize(t);
          GetMem(p, j);
          Registry.ReadBinaryData(t, p^, J);
          for k := 0 to j - 1 do
          begin
            Write(F, IntToHex(byte(p[k]), 2));
            if (k <> j - 1) then
            begin
              Write(F, ',');
              if (k > 0) and ((k mod 25) = 0) then
                Writeln(F, '\');
            end;
          end;
          Freemem(p, j);
          Writeln(F);
        end;
        else
          Writeln(F, '""');
      end;
    end;
    Registry.CloseKey;
    tslValues.Free;
    for i := 0 to tslKeys.Count - 1 do
      ProcessBranch(AKey + '\' + tslKeys[i]);
    tslKeys.Free;
  end;

begin
  if (FKey <> '') then
  begin
    AssignFile(F, AFileName);
    Rewrite(F);
    if (IOResult <> 0) then
      Exit;
    Writeln(F, 'REGEDIT4');
    Registry := TRegistry.Create;
    try
      Registry.Access := AccessValues[FAccess];
      Registry.RootKey := RootKeyValues[FRootKey];
      AKey := CheckKey(FKey) + CheckKey(ASubKey);
      ProcessBranch(AKey);
    finally
      Registry.Free;
      Close(F);
    end;
  end;
end;

 {***************************************************************************}
 { PROCEDURE SetEncryptionParameters                                         }
 {***************************************************************************}

procedure TRegistryEnhanced.SetEncryptionParameters(ABaseKey: string;
  ADecryptKey: integer);
begin
  FBaseKey    := ABaseKey;
  FDecryptKey := ADecryptKey;
end;

 {***************************************************************************}
 { PROCEDURE Register                                                        }
 {***************************************************************************}

procedure Register;
begin
  RegisterComponents('System', [TRegistryEnhanced]);
end;

end.

